#ifndef _M3DMATH_H_
#define _M3DMATH_H_

#define PI    3.141592653589
#define PI180 0.017453292519
#define EPSILON 1e-6
#define DEG2RAD (PI/180.0f)
#define RAD2DEG (180.0f/PI)

// 2-rozmerny vektor
struct tVec2 {
 tVec2():x(0.0f),y(0.0f) {}
 tVec2(float x,float y):x(x),y(y) {}

 float x,y;
};

// 3-rozmerny vektor
struct tVec3 {
 tVec3():x(0.0f),y(0.0f),z(0.0f) {}
 tVec3(float x,float y,float z):x(x),y(y),z(z) {}
 tVec3(const tVec2 &v):x(v.x),y(v.y),z(1.0f) {}
 tVec3(const tVec2 &v,float z):x(v.x),y(v.y),z(z) {}

 const tVec3 operator*(float f) const { return tVec3(x*f,y*f,z*f); }
 const tVec3 operator/(float f) const { return tVec3(x/f,y/f,z/f); }
 const tVec3 operator+(const tVec3 &v) const { return tVec3(x+v.x,y+v.y,z+v.z); }
 const tVec3 operator-(const tVec3 &v) const { return tVec3(x-v.x,y-v.y,z-v.z); }
 const tVec3 operator-() const { return tVec3(-x,-y,-z); }

 float x,y,z;
};

// 4-rozmerny vektor
struct tVec4 {
 tVec4():x(0.0f),y(0.0f),z(0.0f),w(0.0f) {}
 tVec4(float x,float y,float z,float w):x(x),y(y),z(z),w(w) {}
 tVec4(const tVec3 &v):x(v.x),y(v.y),z(v.z),w(1.0f) {}
 tVec4(const tVec3 &v,float w):x(v.x),y(v.y),z(v.z),w(w) {}

 operator float*() { return (float*)&x; }
 operator const float*() const { return (float*)&x; }

 float x,y,z,w;
};

// 16-prvkova matica
struct mat16 {
 mat16() {
  mat[0]=1.0f; mat[4]=0.0f; mat[8]=0.0f; mat[12]=0.0f;
  mat[1]=0.0f; mat[5]=1.0f; mat[9]=0.0f; mat[13]=0.0f;
  mat[2]=0.0f; mat[6]=0.0f; mat[10]=1.0f; mat[14]=0.0f;
  mat[3]=0.0f; mat[7]=0.0f; mat[11]=0.0f; mat[15]=1.0f;
 }

 operator float*() { return (float*)mat; }
 operator const float*() const { return (float*)mat; }

 float mat[16];
};

// trieda premennych a funkcii pre pracu s matematickymi vypoctami
class m3dmath {
public:
 m3dmath();          // konstruktor
 virtual ~m3dmath(); // destruktor

 // vypocet prvkov 3-rozmerneho vektora
 tVec3 Vector(tVec3 vPoint1,tVec3 vPoint2);
 // sucet vektorov
 tVec3 AddVector(tVec3 vVec1,tVec3 vVec2);
 // delenie vektora skalarom
 tVec3 DivideVectorByScalar(tVec3 vVec1,float Scalar);
 // vektorovy sucin dvoch vektorov
 tVec3 Cross(tVec3 vVec1,tVec3 vVec2);
 // normalizacia normaly
 tVec3 Normalize(tVec3 vNormal);
 // vypocet dlzky vektora
 float Length(tVec3 vVec1,tVec3 vVec2);
private:
 tVec3 vResult; // vysledok
 float vMag;    // velkost vektora
};

#endif
