// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
// GL a GLEXT kniznica
#include <GL/gl.h>
#include <GL/glext.h>

// .h subor
#include "mColdCathode.h"

// objekt predstavujuci neonove svietidlo, jedna sa o jednoduchy 8-hran
// s uzatvorenou celnou aj zadnou stenou
GLfloat CCvertex[54]={ 0.000000,-1.000000, 1.000000,
                       0.707106,-0.707106, 1.000000,
                       1.000000, 0.000000, 1.000000,
                       0.707106, 0.707106, 1.000000,
                       0.000000, 1.000000, 1.000000,
                      -0.707106, 0.707106, 1.000000,
                      -1.000000, 0.000000, 1.000000,
                      -0.707106,-0.707106, 1.000000,
                       0.000000, 0.000000, 1.000000,

                       0.000000,-1.000000, 0.000000,
                       0.707106,-0.707106, 0.000000,
                       1.000000, 0.000000, 0.000000,
                       0.707106, 0.707106, 0.000000,
                       0.000000, 1.000000, 0.000000,
                      -0.707106, 0.707106, 0.000000,
                      -1.000000, 0.000000, 0.000000,
                      -0.707106,-0.707106, 0.000000,
                       0.000000, 0.000000, 0.000000};
// indexy - spajaju vzdy 3 vrcholy tak, aby bol vytvoreny trojuholnik,
// najskor sa vytvori celna stena, potom 8 stien a nakoniec zadna stena
GLuint CCindex[24+48+24]={0,1,8, 1,2,8, 2,3,8, 3,4,8, 4,5,8, 5,6,8, 6,7,8, 7,0,8,
                          0,9,1,9,10,1, 1,10,2,10,11,2, 2,11,3,11,12,3, 3,12,4,12,13,4,
                          4,13,5,13,14,5, 5,14,6,14,15,6, 6,15,7,15,16,7, 7,16,0,16,9,0,
                          10,9,17, 11,10,17, 12,11,17, 13,12,17, 14,13,17, 15,14,17, 16,15,17, 9,16,17};

// konstruktor *************************************************************
mColdCathode::mColdCathode()
{

}

// destruktor **************************************************************
mColdCathode::~mColdCathode()
{

}

// inicializacia neonoveho svietidla ***************************************
void mColdCathode::init(const tVec4 C,const float R)
{
 // ulozenie farby a polomeru trubice
 color=C;
 radius=R;
 // nahratie ShaderProgramov
 lamp_vp.PRGload("data/shaders/cclamp.vp");
 lamp_fp.PRGload("data/shaders/cclamp.fp");
}

// vykreslenie neonoveho svietidla *****************************************
void mColdCathode::render(const tVec3 C)
{
 // zapnutie Vertex a Fragment Programu pre LampShader
 lamp_vp.PRGenable(); lamp_vp.PRGbind();
 lamp_fp.PRGenable(); lamp_fp.PRGbind();

 // presun udajov do ShaderProgramov
 lamp_vp.PRGlocal_param(0,tVec4(pos[0]));                     // 1. koniec trubice
 lamp_vp.PRGlocal_param(1,tVec4(pos[1]));                     // 2. koniec trubice
 lamp_vp.PRGlocal_param(2,tVec4(C));                          // suradnice kamery
 lamp_fp.PRGlocal_param(0,tVec4(1.0f/radius,0.0f,0.0f,0.0f)); // polomer trubice

 glPushMatrix();

  A[0]=pos[1]-pos[0];            // A[0] = vektor z pos[0] do pos[1]
  A[0]=mm.Normalize(A[0]);
  B.x=0.0f; B.y=0.0f; B.z=-1.0f; // B = konstantny vektor v smere osi Z
  A[1]=mm.Cross(A[0],B);         // A[1] = vektor kolmy na A[0] a B
  A[1]=mm.Normalize(A[1]);
  A[2]=mm.Cross(A[0],A[1]);      // A[2] = vektor kolmy na A[0] a A[1]
  A[2]=mm.Normalize(A[2]);

  // ulozenie vektorov do pomocnej matice M
  // A[0] 0    rotacia
  // A[1] 0    rotacia
  // A[2] 0    rotacia
  // pos0 1    posun
  // tymto sa vytvori modelview matica na zaklade
  // informacie o polohe 2 vrcholov trubice
  M[0]=A[0].x;    M[1]=A[0].y;    M[2]=A[0].z;    M[3]=0.0f;
  M[4]=A[1].x;    M[5]=A[1].y;    M[6]=A[1].z;    M[7]=0.0f;
  M[8]=A[2].x;    M[9]=A[2].y;    M[10]=A[2].z;   M[11]=0.0f;
  M[12]=pos[0].x; M[13]=pos[0].y; M[14]=pos[0].z; M[15]=1.0f;

  // uprava pomocnej transformacnej matice
  glMatrixMode(GL_MATRIX0_ARB);
  glLoadMatrixf(M);
  glRotatef(90.0f,0.0f,1.0f,0.0f);
  glTranslatef(0.0f,0.0f,-radius);
  glScalef(radius,radius,mm.Length(pos[1],pos[0])+radius*2.0f);

  // uprava modelovej matice
  glMatrixMode(GL_MODELVIEW);
  glMultMatrixf(M);
  glRotatef(90.0f,0.0f,1.0f,0.0f);
  glTranslatef(0.0f,0.0f,-radius);
  glScalef(radius,radius,mm.Length(pos[1],pos[0])+radius*2.0f);

  // vykreslenie objektu svetelnej trubice
  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(3,GL_FLOAT,0,CCvertex);
  glDrawElements(GL_TRIANGLES,24+48+24,GL_UNSIGNED_INT,CCindex);
  glDisableClientState(GL_VERTEX_ARRAY);

 glPopMatrix();

 // vypnutie Vertex a Fragment Programu pre LampShader
 lamp_fp.PRGdisable();
 lamp_vp.PRGdisable();
}
