// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <string.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mOGL.h"

// konstruktor *************************************************************
mOGL::mOGL()
{
 Cnum=0; // cislo pouzitej OpenGL kamery

 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 StartInfoPos=10.0f; // pozicia vypisu 1. textu pocas startu aplikacie

 t_a=0; t_b=0; t_c=0; // pomocne premenne

 lighting_onoff=0;  // osvetlenie
 help_onoff=0;      // napoveda
 scene_draw_type=0; // standardny pohlad na scenu
 pause_motion=0;    // casovo zavisly pohyb
 normals_onoff=0;   // vypnutie vykreslovania normal

 SOvertex=NULL; SOnormal=NULL; SOtangent=NULL; SObinormal=NULL; SOuv=NULL;
 SOvertex_num=0;
 cc_lamp[0]=NULL; cc_lamp[1]=NULL; cc_lamp[2]=NULL; cc_lamp[3]=NULL;
}

// destruktor **************************************************************
mOGL::~mOGL()
{
 // uvolnenie alokovanej pamate
 if (cc_lamp[0]!=NULL) delete cc_lamp[0];
 if (cc_lamp[1]!=NULL) delete cc_lamp[1];
 if (cc_lamp[2]!=NULL) delete cc_lamp[2];
 if (cc_lamp[3]!=NULL) delete cc_lamp[3];

 if (SOvertex!=NULL) delete SOvertex;
 if (SOnormal!=NULL) delete SOnormal;
 if (SOtangent!=NULL) delete SOtangent;
 if (SObinormal!=NULL) delete SObinormal;
 if (SOuv!=NULL) delete SOuv;
}

// vypis info pocas startu aplikacie ***************************************
void mOGL::PrintStartInfo(char *infotext)
{
 ogl_Text.OpenDrawString();
 ogl_Text.DrawString(10.0f,StartInfoPos,infotext);
 ogl_Text.CloseDrawString();

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 // dalsi text bude vypisany na novu poziciu
 StartInfoPos+=20.0f;
 // pauza aby bol text aspon chvilku viditelny
 SDL_Delay(100);
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
bool mOGL::gl_initialize(void)
{
 // nastavenie 8-mich vseobecnych materialov
 for (t_a=0;t_a<8;t_a++) {
  ogl_Material[t_a].amb[0]=0.1f; ogl_Material[t_a].amb[1]=0.1f;
  ogl_Material[t_a].amb[2]=0.1f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.8f; ogl_Material[t_a].dif[1]=0.8f;
  ogl_Material[t_a].dif[2]=0.8f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=128.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 8-mich vseobecnych svetiel
 for (t_a=0;t_a<8;t_a++) {
  ogl_Light[t_a].amb[0]=0.1f; ogl_Light[t_a].amb[1]=0.1f;
  ogl_Light[t_a].amb[2]=0.1f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.9f; ogl_Light[t_a].dif[1]=0.9f;
  ogl_Light[t_a].dif[2]=0.9f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.9f; ogl_Light[t_a].spe[1]=0.9f;
  ogl_Light[t_a].spe[2]=0.9f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=-10.0f; ogl_Light[t_a].pos[1]=10.0f;
  ogl_Light[t_a].pos[2]=10.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.0f,0.0f,0.0f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 //glClearStencil(0);               // stencil zasobnik bude vymazavany nastavenim na 0

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,1.0f);

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT,ogl_Material[0].amb);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,ogl_Material[0].dif);
 glMaterialfv(GL_FRONT,GL_SPECULAR,ogl_Material[0].spe);
 glMaterialfv(GL_FRONT,GL_EMISSION,ogl_Material[0].emi);
 glMaterialf(GL_FRONT,GL_SHININESS,ogl_Material[0].shi);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);
 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 fprintf(stderr,"----------------------------------------------------------\n");

 // vymazanie zasobnika farieb, hlbky a stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // inicializacia vypisovania OpenGL textu
 if (ogl_Text.TextInitialize()) {
  fprintf(stderr,"Chyba inicializacie fontu!\n"); return(1);
 }
 // potvrdenie startu OpenGL
 PrintStartInfo("Nastavenie OpenGL stavoveho stroja...");
 PrintStartInfo("OK");

 // zameriavaci kriz
 PrintStartInfo("Nahratie textury zameriavacieho kriza...");
 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 PrintStartInfo("OK");

 // textury
 PrintStartInfo("Nahratie textur...");
 if (ogl_Texture.LoadTexture(0,"data/floor_nmap.tga",TEXTURE_TRILINEAR|TEXTURE_MIPMAP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(1,"data/wall_nmap.tga",TEXTURE_TRILINEAR|TEXTURE_MIPMAP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.Load1DTexture(2,"data/cclamp0.tga",TEXTURE_CLAMP_TO_EDGE|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.Load1DTexture(3,"data/cclamp1.tga",TEXTURE_CLAMP_TO_EDGE|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.Load1DTexture(4,"data/cclamp2.tga",TEXTURE_CLAMP_TO_EDGE|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.Load1DTexture(5,"data/cclamp3.tga",TEXTURE_CLAMP_TO_EDGE|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(6,"data/floor.tga",TEXTURE_TRILINEAR|TEXTURE_MIPMAP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(7,"data/wall.tga",TEXTURE_TRILINEAR|TEXTURE_MIPMAP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 PrintStartInfo("OK");

 PrintStartInfo("Test rozsireni...");
 // 1. krok - test ci karta podporuje rozsirenia............................
 light_vp.CheckExtension("GL_ARB_multitexture");
 light_vp.CheckExtension("GL_ARB_vertex_program");
 light_fp.CheckExtension("GL_ARB_fragment_program");
 PrintStartInfo("OK");

 PrintStartInfo("Priprava osvetleneho objektu...");
 // 2. krok - vytovrenie objektu, ktory bude osvetelny neonovymi svietidlami
 // bude sa jednat o dve steny a podlahu, ktore budu spolu predstavovat
 // jednoduchu chodbu ......................................................
 SOvertex_num=18;
 SOvertex=new tVec3[SOvertex_num];   // pole vrcholov
 SOnormal=new tVec3[SOvertex_num];   // pole normal
 SOtangent=new tVec3[SOvertex_num];  // pole tangent-vektorov - v smere U texturovacich suradnic
 SObinormal=new tVec3[SOvertex_num]; // pole binormal-vektorov - v smere V texturovacich suradnic
 SOuv=new tVec2[SOvertex_num];       // pole UV texturovacich suradnic
 // podlaha
 SOvertex[0].x=-4.0f; SOvertex[0].y=0.0f; SOvertex[0].z=10.0f;
 SOvertex[1].x=4.0f;  SOvertex[1].y=0.0f; SOvertex[1].z=10.0f;
 SOvertex[2].x=-4.0f; SOvertex[2].y=0.0f; SOvertex[2].z=-10.0f;
 SOvertex[3].x=4.0f;  SOvertex[3].y=0.0f; SOvertex[3].z=10.0f;
 SOvertex[4].x=4.0f;  SOvertex[4].y=0.0f; SOvertex[4].z=-10.0f;
 SOvertex[5].x=-4.0f; SOvertex[5].y=0.0f; SOvertex[5].z=-10.0f;

 SOnormal[0].x=0.0f; SOnormal[0].y=1.0f; SOnormal[0].z=0.0f;
 SOnormal[1].x=0.0f; SOnormal[1].y=1.0f; SOnormal[1].z=0.0f;
 SOnormal[2].x=0.0f; SOnormal[2].y=1.0f; SOnormal[2].z=0.0f;
 SOnormal[3].x=0.0f; SOnormal[3].y=1.0f; SOnormal[3].z=0.0f;
 SOnormal[4].x=0.0f; SOnormal[4].y=1.0f; SOnormal[4].z=0.0f;
 SOnormal[5].x=0.0f; SOnormal[5].y=1.0f; SOnormal[5].z=0.0f;

 SOtangent[0].x=1.0f; SOtangent[0].y=0.0f; SOtangent[0].z=0.0f;
 SOtangent[1].x=1.0f; SOtangent[1].y=0.0f; SOtangent[1].z=0.0f;
 SOtangent[2].x=1.0f; SOtangent[2].y=0.0f; SOtangent[2].z=0.0f;
 SOtangent[3].x=1.0f; SOtangent[3].y=0.0f; SOtangent[3].z=0.0f;
 SOtangent[4].x=1.0f; SOtangent[4].y=0.0f; SOtangent[4].z=0.0f;
 SOtangent[5].x=1.0f; SOtangent[5].y=0.0f; SOtangent[5].z=0.0f;

 SObinormal[0].x=0.0f; SObinormal[0].y=0.0f; SObinormal[0].z=-1.0f;
 SObinormal[1].x=0.0f; SObinormal[1].y=0.0f; SObinormal[1].z=-1.0f;
 SObinormal[2].x=0.0f; SObinormal[2].y=0.0f; SObinormal[2].z=-1.0f;
 SObinormal[3].x=0.0f; SObinormal[3].y=0.0f; SObinormal[3].z=-1.0f;
 SObinormal[4].x=0.0f; SObinormal[4].y=0.0f; SObinormal[4].z=-1.0f;
 SObinormal[5].x=0.0f; SObinormal[5].y=0.0f; SObinormal[5].z=-1.0f;

 SOuv[0].x=0.0f; SOuv[0].y=0.0f;
 SOuv[1].x=8.0f; SOuv[1].y=0.0f;
 SOuv[2].x=0.0f; SOuv[2].y=20.0f;
 SOuv[3].x=8.0f; SOuv[3].y=0.0f;
 SOuv[4].x=8.0f; SOuv[4].y=20.0f;
 SOuv[5].x=0.0f; SOuv[5].y=20.0f;
 // lava stena
 SOvertex[6].x=-4.0f;  SOvertex[6].y=0.0f;  SOvertex[6].z=10.0f;
 SOvertex[7].x=-4.0f;  SOvertex[7].y=0.0f;  SOvertex[7].z=-10.0f;
 SOvertex[8].x=-4.0f;  SOvertex[8].y=3.0f;  SOvertex[8].z=10.0f;
 SOvertex[9].x=-4.0f;  SOvertex[9].y=0.0f;  SOvertex[9].z=-10.0f;
 SOvertex[10].x=-4.0f; SOvertex[10].y=3.0f; SOvertex[10].z=-10.0f;
 SOvertex[11].x=-4.0f; SOvertex[11].y=3.0f; SOvertex[11].z=10.0f;

 SOnormal[6].x=1.0f;  SOnormal[6].y=0.0f;  SOnormal[6].z=0.0f;
 SOnormal[7].x=1.0f;  SOnormal[7].y=0.0f;  SOnormal[7].z=0.0f;
 SOnormal[8].x=1.0f;  SOnormal[8].y=0.0f;  SOnormal[8].z=0.0f;
 SOnormal[9].x=1.0f;  SOnormal[9].y=0.0f;  SOnormal[9].z=0.0f;
 SOnormal[10].x=1.0f; SOnormal[10].y=0.0f; SOnormal[10].z=0.0f;
 SOnormal[11].x=1.0f; SOnormal[11].y=0.0f; SOnormal[11].z=0.0f;

 SOtangent[6].x=0.0f;  SOtangent[6].y=0.0f;  SOtangent[6].z=-1.0f;
 SOtangent[7].x=0.0f;  SOtangent[7].y=0.0f;  SOtangent[7].z=-1.0f;
 SOtangent[8].x=0.0f;  SOtangent[8].y=0.0f;  SOtangent[8].z=-1.0f;
 SOtangent[9].x=0.0f;  SOtangent[9].y=0.0f;  SOtangent[9].z=-1.0f;
 SOtangent[10].x=0.0f; SOtangent[10].y=0.0f; SOtangent[10].z=-1.0f;
 SOtangent[11].x=0.0f; SOtangent[11].y=0.0f; SOtangent[11].z=-1.0f;

 SObinormal[6].x=0.0f;  SObinormal[6].y=1.0f;  SObinormal[6].z=0.0f;
 SObinormal[7].x=0.0f;  SObinormal[7].y=1.0f;  SObinormal[7].z=0.0f;
 SObinormal[8].x=0.0f;  SObinormal[8].y=1.0f;  SObinormal[8].z=0.0f;
 SObinormal[9].x=0.0f;  SObinormal[9].y=1.0f;  SObinormal[9].z=0.0f;
 SObinormal[10].x=0.0f; SObinormal[10].y=1.0f; SObinormal[10].z=0.0f;
 SObinormal[11].x=0.0f; SObinormal[11].y=1.0f; SObinormal[11].z=0.0f;

 SOuv[6].x=0.0f;   SOuv[6].y=0.0f;
 SOuv[7].x=20.0f;  SOuv[7].y=0.0f;
 SOuv[8].x=0.0f;   SOuv[8].y=3.0f;
 SOuv[9].x=20.0f;  SOuv[9].y=0.0f;
 SOuv[10].x=20.0f; SOuv[10].y=3.0f;
 SOuv[11].x=0.0f;  SOuv[11].y=3.0f;
 // prava stena
 SOvertex[12].x=4.0f; SOvertex[12].y=0.0f; SOvertex[12].z=-10.0f;
 SOvertex[13].x=4.0f; SOvertex[13].y=0.0f; SOvertex[13].z=10.0f;
 SOvertex[14].x=4.0f; SOvertex[14].y=3.0f; SOvertex[14].z=-10.0f;
 SOvertex[15].x=4.0f; SOvertex[15].y=0.0f; SOvertex[15].z=10.0f;
 SOvertex[16].x=4.0f; SOvertex[16].y=3.0f; SOvertex[16].z=10.0f;
 SOvertex[17].x=4.0f; SOvertex[17].y=3.0f; SOvertex[17].z=-10.0f;

 SOnormal[12].x=-1.0f; SOnormal[12].y=0.0f; SOnormal[12].z=0.0f;
 SOnormal[13].x=-1.0f; SOnormal[13].y=0.0f; SOnormal[13].z=0.0f;
 SOnormal[14].x=-1.0f; SOnormal[14].y=0.0f; SOnormal[14].z=0.0f;
 SOnormal[15].x=-1.0f; SOnormal[15].y=0.0f; SOnormal[15].z=0.0f;
 SOnormal[16].x=-1.0f; SOnormal[16].y=0.0f; SOnormal[16].z=0.0f;
 SOnormal[17].x=-1.0f; SOnormal[17].y=0.0f; SOnormal[17].z=0.0f;

 SOtangent[12].x=0.0f; SOtangent[12].y=0.0f; SOtangent[12].z=1.0f;
 SOtangent[13].x=0.0f; SOtangent[13].y=0.0f; SOtangent[13].z=1.0f;
 SOtangent[14].x=0.0f; SOtangent[14].y=0.0f; SOtangent[14].z=1.0f;
 SOtangent[15].x=0.0f; SOtangent[15].y=0.0f; SOtangent[15].z=1.0f;
 SOtangent[16].x=0.0f; SOtangent[16].y=0.0f; SOtangent[16].z=1.0f;
 SOtangent[17].x=0.0f; SOtangent[17].y=0.0f; SOtangent[17].z=1.0f;

 SObinormal[12].x=0.0f; SObinormal[12].y=1.0f; SObinormal[12].z=0.0f;
 SObinormal[13].x=0.0f; SObinormal[13].y=1.0f; SObinormal[13].z=0.0f;
 SObinormal[14].x=0.0f; SObinormal[14].y=1.0f; SObinormal[14].z=0.0f;
 SObinormal[15].x=0.0f; SObinormal[15].y=1.0f; SObinormal[15].z=0.0f;
 SObinormal[16].x=0.0f; SObinormal[16].y=1.0f; SObinormal[16].z=0.0f;
 SObinormal[17].x=0.0f; SObinormal[17].y=1.0f; SObinormal[17].z=0.0f;

 SOuv[12].x=0.0f;  SOuv[12].y=0.0f;
 SOuv[13].x=20.0f; SOuv[13].y=0.0f;
 SOuv[14].x=0.0f;  SOuv[14].y=3.0f;
 SOuv[15].x=20.0f; SOuv[15].y=0.0f;
 SOuv[16].x=20.0f; SOuv[16].y=3.0f;
 SOuv[17].x=0.0f;  SOuv[17].y=3.0f;

 // objekt bude ulozeny v display liste
 SOdl_id=glGenLists(4);
 // klasicke vykreslenie objektu (podlaha) - DL 0
 glNewList(SOdl_id,GL_COMPILE);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glVertexPointer(3,GL_FLOAT,0,SOvertex);
  glNormalPointer(GL_FLOAT,0,SOnormal);
  glTexCoordPointer(2,GL_FLOAT,0,SOuv);
  glDrawArrays(GL_TRIANGLES,0,6);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
 glEndList();
 // prenesenie poli do ARB atributov neskor odovzdavanych
 // do VertexProgramu - DL 1
 glNewList(SOdl_id+1,GL_COMPILE);
  glEnableVertexAttribArrayARB(0);
  glEnableVertexAttribArrayARB(1);
  glEnableVertexAttribArrayARB(2);
  glEnableVertexAttribArrayARB(3);
  glEnableVertexAttribArrayARB(4);
  glVertexAttribPointerARB(0,3,GL_FLOAT,0,0,SOvertex);
  glVertexAttribPointerARB(1,3,GL_FLOAT,0,0,SOnormal);
  glVertexAttribPointerARB(2,3,GL_FLOAT,0,0,SOtangent);
  glVertexAttribPointerARB(3,3,GL_FLOAT,0,0,SObinormal);
  glVertexAttribPointerARB(4,2,GL_FLOAT,0,0,SOuv);
  glDrawArrays(GL_TRIANGLES,0,6);
  glDisableVertexAttribArrayARB(4);
  glDisableVertexAttribArrayARB(3);
  glDisableVertexAttribArrayARB(2);
  glDisableVertexAttribArrayARB(1);
  glDisableVertexAttribArrayARB(0);
 glEndList();
 // klasicke vykreslenie objektu (steny) - DL 2
 glNewList(SOdl_id+2,GL_COMPILE);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glVertexPointer(3,GL_FLOAT,0,SOvertex);
  glNormalPointer(GL_FLOAT,0,SOnormal);
  glTexCoordPointer(2,GL_FLOAT,0,SOuv);
  glDrawArrays(GL_TRIANGLES,6,12);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
 glEndList();
 // prenesenie poli do ARB atributov neskor odovzdavanych
 // do VertexProgramu - DL 3
 glNewList(SOdl_id+3,GL_COMPILE);
  glEnableVertexAttribArrayARB(0);
  glEnableVertexAttribArrayARB(1);
  glEnableVertexAttribArrayARB(2);
  glEnableVertexAttribArrayARB(3);
  glEnableVertexAttribArrayARB(4);
  glVertexAttribPointerARB(0,3,GL_FLOAT,0,0,SOvertex);
  glVertexAttribPointerARB(1,3,GL_FLOAT,0,0,SOnormal);
  glVertexAttribPointerARB(2,3,GL_FLOAT,0,0,SOtangent);
  glVertexAttribPointerARB(3,3,GL_FLOAT,0,0,SObinormal);
  glVertexAttribPointerARB(4,2,GL_FLOAT,0,0,SOuv);
  glDrawArrays(GL_TRIANGLES,6,12);
  glDisableVertexAttribArrayARB(4);
  glDisableVertexAttribArrayARB(3);
  glDisableVertexAttribArrayARB(2);
  glDisableVertexAttribArrayARB(1);
  glDisableVertexAttribArrayARB(0);
 glEndList();
 PrintStartInfo("OK");

 PrintStartInfo("Priprava neonovych svietidiel...");
 // 3. krok - priprava neonovych svietidiel ................................
 cc_lamp[0]=new mColdCathode;
 cc_lamp[0]->init(tVec4(0.83f,0.47f,0.47f,2.0f),0.1f); // cervene
 cc_lamp[1]=new mColdCathode;
 cc_lamp[1]->init(tVec4(0.66f,0.83f,0.47f,2.0f),0.1f); // zelene
 cc_lamp[2]=new mColdCathode;
 cc_lamp[2]->init(tVec4(0.47f,0.83f,0.83f,2.0f),0.1f); // modre
 cc_lamp[3]=new mColdCathode;
 cc_lamp[3]->init(tVec4(0.72f,0.72f,0.72f,2.0f),0.1f); // biele
 PrintStartInfo("OK");

 PrintStartInfo("Nahratie shader programov...");
 // 4. krok - nahratie Vertex a Fragment Programu pre LightShader ..........
 light_vp.PRGload("data/shaders/light.vp");
 light_fp.PRGload("data/shaders/light.fp");
 PrintStartInfo("OK");

 // startovaci cas
 mtime=0.0f;

 // priprava pouzitia rozsireni
 light_vp.Ext_init();

 // koniec pociatocnych nastaveni, zaciatok interakcie pozorovatela
 PrintStartInfo("START");

 // nastavenie farby a velkosti neskor vypisovaneho textu
 ogl_Text.SetTextColor(1.0f,1.0f,1.0f);
 ogl_Text.SetTextSize(10);

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
int mOGL::gl_draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glMultMatrixf(ogl_Camera[Cnum].matrix);
 else {
  glRotatef(45.0f,1.0f,0.0f,0.0f);  // zapnutie specialneho pohladu na scenu
  glRotatef(-30.0f,0.0f,1.0f,0.0f);
 }
 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glTranslatef(ogl_Camera[Cnum].posx,ogl_Camera[Cnum].posy,ogl_Camera[Cnum].posz);
 else glTranslatef(-7.5f,-10.0f,-15.0f); // v pripade specialneho pohladu na scenu

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-ogl_Camera[0].posx;
 //ogl_Light[0].pos[1]=-ogl_Camera[0].posy;
 //ogl_Light[0].pos[2]=-ogl_Camera[0].posz;

 // umiestenenie svetla na preddefinovanu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // zapnutie texturovacej jednotky0
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D);
 // zapnutie texturovacej jednotky1
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D);

 // 1. zmena polohy koncov svietidiel - svetelnych trubic ..................
 cc_lamp[0]->pos[0]=tVec3(-3.5f,1.5f+(-1.0f*cos(mtime)), 2.5f+( 1.0f*sin(mtime)));
 cc_lamp[0]->pos[1]=tVec3(-3.5f,1.5f+( 1.0f*cos(mtime)), 2.5f+(-1.0f*sin(mtime)));
 cc_lamp[1]->pos[0]=tVec3(-3.5f,1.5f+( 1.0f*cos(mtime)),-2.5f+(-1.0f*sin(mtime)));
 cc_lamp[1]->pos[1]=tVec3(-3.5f,1.5f+(-1.0f*cos(mtime)),-2.5f+( 1.0f*sin(mtime)));
 cc_lamp[2]->pos[0]=tVec3( 3.5f,1.5f+( 1.0f*cos(mtime)), 2.5f+(-1.0f*sin(mtime)));
 cc_lamp[2]->pos[1]=tVec3( 3.5f,1.5f+(-1.0f*cos(mtime)), 2.5f+( 1.0f*sin(mtime)));
 cc_lamp[3]->pos[0]=tVec3( 3.5f,1.5f+( 1.0f*cos(mtime)),-2.5f+(-1.0f*sin(mtime)));
 cc_lamp[3]->pos[1]=tVec3( 3.5f,1.5f+(-1.0f*cos(mtime)),-2.5f+( 1.0f*sin(mtime)));

 // 2. vykreslenie objektu osvetleneho neonovymi svietidlami ...............
 glEnable(GL_BLEND);
 glBlendFunc(GL_ONE,GL_ONE);

 // vyuzije sa prislusny Vertex a Fragment Program pre LightShader
 light_vp.PRGenable(); light_vp.PRGbind();
 light_fp.PRGenable(); light_fp.PRGbind();
 // podlaha
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[0]);
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[6]);
 // svetlo vyzarovane 1. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[0]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[0]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[0]->color);
 glCallList(SOdl_id+1);

 // svetlo vyzarovane 2. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[1]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[1]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[1]->color);
 glCallList(SOdl_id+1);

 // svetlo vyzarovane 3. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[2]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[2]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[2]->color);
 glCallList(SOdl_id+1);

 // svetlo vyzarovane 4. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[3]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[3]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[3]->color);
 glCallList(SOdl_id+1);

 // steny
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[1]);
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[7]);
 // svetlo vyzarovane 1. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[0]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[0]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[0]->color);
 glCallList(SOdl_id+3);

 // svetlo vyzarovane 2. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[1]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[1]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[1]->color);
 glCallList(SOdl_id+3);

 // svetlo vyzarovane 3. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[2]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[2]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[2]->color);
 glCallList(SOdl_id+3);

 // svetlo vyzarovane 4. neonovym svietidlom
 light_vp.PRGlocal_param(0,tVec4(cc_lamp[3]->pos[0],0.0f));
 light_vp.PRGlocal_param(1,tVec4(cc_lamp[3]->pos[1],0.0f));
 light_fp.PRGlocal_param(0,cc_lamp[3]->color);
 glCallList(SOdl_id+3);

 // vypnutie texturovacej jednotky1
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D);
 // dalej sa bude pouzivat iba jednotka0
 glActiveTextureARB(GL_TEXTURE0_ARB);

 // ukoncenie vykonavania Vertex a Fragment Programu LightShadera
 light_fp.PRGdisable();
 light_vp.PRGdisable();

 glDisable(GL_BLEND);

 // 3. vykreslenie neonovych svietidiel ....................................
 glEnable(GL_TEXTURE_1D);

 glEnable(GL_BLEND);
 glBlendFunc(GL_ONE,GL_ONE);
 glDepthMask(GL_FALSE);
 // prevzatie suradnic kamery
 cc_camera.x=-ogl_Camera[Cnum].posx;
 cc_camera.y=-ogl_Camera[Cnum].posy;
 cc_camera.z=-ogl_Camera[Cnum].posz;
 // vykreslenie objektov neonovych trubic
 glBindTexture(GL_TEXTURE_1D,ogl_Texture.texture_id[2]);
 cc_lamp[0]->render(cc_camera);
 glBindTexture(GL_TEXTURE_1D,ogl_Texture.texture_id[3]);
 cc_lamp[1]->render(cc_camera);
 glBindTexture(GL_TEXTURE_1D,ogl_Texture.texture_id[4]);
 cc_lamp[2]->render(cc_camera);
 glBindTexture(GL_TEXTURE_1D,ogl_Texture.texture_id[5]);
 cc_lamp[3]->render(cc_camera);

 glDepthMask(GL_TRUE);
 glDisable(GL_BLEND);

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair(ogl_Camera[Cnum].scr_w,ogl_Camera[Cnum].scr_h);

 if (help_onoff) {
  // vypis textu - help
  ogl_Text.OpenDrawString();

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-20.0f, "ESC ukoncenie aplikacie");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-60.0f, "MYS                rotacia v danom smere");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-80.0f, "SIPKY              posun dopredu,dozadu,vlavo,vpravo");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-100.0f,"SHIFT              zrychlenie posunu");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-140.0f,"R     reinicializacia pohladu kamery");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-160.0f,"M     vyplnene trojuholniky/iba hrany trojuholnikov");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-180.0f,"P     zapnutie/vypnutie casovo zavisleho pohybu objektov");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-200.0f,"N     zapnutie/vypnutie vykreslovania normal objektov");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-240.0f,"H     zapnutie/vypnutie zobrazenia helpu");

  ogl_Text.CloseDrawString();
 }
 else {
  // vypis textu - poloha kamery
  ogl_Text.OpenDrawString();

  sprintf(ogl_Text.numberbuf,"x:%6.2f",-ogl_Camera[Cnum].posx);
  ogl_Text.DrawNumber(10.0f,30.0f);
  sprintf(ogl_Text.numberbuf,"y:%6.2f",-ogl_Camera[Cnum].posy);
  ogl_Text.DrawNumber(10.0f,20.0f);
  sprintf(ogl_Text.numberbuf,"z:%6.2f",-ogl_Camera[Cnum].posz);
  ogl_Text.DrawNumber(10.0f,10.0f);

  // vypis textu - pocet obrazkov za sekundu
  sprintf(ogl_Text.numberbuf,"%i fps",(GLuint) ogl_Camera[Cnum].ogl_fps);
  //sprintf(ogl_Text.numberbuf,"%f fps",ogl_Camera[Cnum].GetCameraTime()*1000.0f);
  ogl_Text.DrawNumber(10.0f,ogl_Camera[Cnum].scr_h-20.0f);

  ogl_Text.CloseDrawString();
 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mOGL::gl_iddle(void)
{
 if (pause_motion==0) {
  mtime+=ogl_Camera[Cnum].GetCameraTime();
 }

 return(0);
}

// zmena sposobu zobrazenia objektov ***************************************
void mOGL::change_PolygonMode(void)
{
 glGetIntegerv(GL_POLYGON_MODE,&ogl_State[0]);
 // INFO: vypis nacitaneho stavu
 //fprintf(stderr,"ogl_State=%d\n",ogl_State[0]);

 if (ogl_State[0]==GL_LINE) glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 SDL_Delay(200);
}

// zap/vyp casovo zavisleho pohybu objektov ********************************
void mOGL::change_PauseMotion(void)
{
 if (pause_motion==0) pause_motion=1;
 else pause_motion=0;

 SDL_Delay(200);
}

// zap/vyp vypisu helpu ****************************************************
void mOGL::change_HelpOnOff(void)
{
 if (help_onoff==0) help_onoff=1;
 else help_onoff=0;

 SDL_Delay(200);
}

// zap/vyp vykreslovania normal objektov ***********************************
void mOGL::change_NormalsOnOff(void)
{
 if (normals_onoff==0) normals_onoff=1;
 else normals_onoff=0;

 SDL_Delay(200);
}
